#include <FastLED.h>

#define NUM_LEDS 64

#define DATA_PIN 5
#define DATA_PIN_2 6
#define CLOCK_PIN 13

#define LIMIT_INF 0
#define LIMIT_SUP 7

// Arduino pin numbers
const int SW_pin_joystick0 = 2; // digital pin connected to switch output
const int X_pin_joystick0 = 0; // analog pin connected to X output
const int Y_pin_joystick0 = 1; // analog pin connected to Y output

const int SW_pin_joystick1 = 4; // digital pin connected to switch output
const int X_pin_joystick1 = 2; // analog pin connected to X output
const int Y_pin_joystick1 = 3; // analog pin connected to Y output

const int buzzerPin = 8;

int cursorX = 0;
int cursorY = 0;

int lastCursorX = -1;
int lastCursorY = -1;

int player = 0;

//Define the array of leds
void resetGame();
CRGB ledsP1Boats[NUM_LEDS];
CRGB ledsP1Hits[NUM_LEDS];

CRGB ledsP2Boats[NUM_LEDS];
CRGB ledsP2Hits[NUM_LEDS];

CRGB ledsP1[NUM_LEDS];
CRGB ledsP2[NUM_LEDS];

CRGB boat = CRGB::Green;
CRGB empty = CRGB::Black;
CRGB hit  = CRGB::Red;
CRGB miss  = CRGB::Green;
CRGB cursor = CRGB::Blue;

CRGB cPlayer0 = CRGB::Blue;
CRGB cPlayer1 = CRGB::Blue;

void clearLeds(CRGB *mat) {
  int i = 0;
  for(i = 0; i < 64; i++) {
    mat[i] = empty;
    mat[i] = empty;
    mat[i] = empty;
    mat[i] = empty;
  }
}

void resetGame() {
  // set boats
  int i = 0;
  Serial.print("called\n");
  clearLeds(ledsP1Boats);
  clearLeds(ledsP2Boats);
  clearLeds(ledsP1Hits);
  clearLeds(ledsP2Hits);

  for(i = 0; i < 4; i++) {
    int posX = random(8);
    int posY = random(8);
    while(ledsP1Boats[8 * posX + posY] != empty) {
        posX = random(8);
        posY = random(8);
    }
    ledsP1Boats[8 * posX + posY] = boat;

    posX = random(8);
    posY = random(8);
    while(ledsP2Boats[8 * posX + posY] != empty) {
        posX = random(8);
        posY = random(8);
    }
    ledsP2Boats[8 * posX + posY] = boat;
  }

  tone(buzzerPin, 250);
  delay(250);
  tone(buzzerPin, 450);
  delay(250);
  tone(buzzerPin, 650);
  delay(250);
  noTone(buzzerPin);
}

void setup() {
  pinMode(SW_pin_joystick0 , INPUT);
  digitalWrite(SW_pin_joystick0 , HIGH);
  pinMode(SW_pin_joystick1 , INPUT);
  digitalWrite(SW_pin_joystick1 , HIGH);
  Serial.begin(9600);
  
  resetGame();

  LEDS.addLeds<WS2812, DATA_PIN, RGB>(ledsP1, NUM_LEDS);
  LEDS.addLeds<WS2812, DATA_PIN_2, RGB>(ledsP2, NUM_LEDS);
  LEDS.setBrightness(84);


  EIMSK &= ~(1 << INT0);
  EICRA |= (1 << ISC00);
  EIFR &= ~(1 << INTF0); 
  EIMSK |= (1 << INT0);
  PCICR |= (1 << PCIE2);
  PCMSK2 |= (1 << PCINT20); // Turns on PCINT20 (PD4)
  sei();
}

long timerPD2 = 0;
ISR(INT0_vect){    //INTO_vect indicates that ISR() handles an INT0 external interrupt 
  if ((PIND & (1 << PD2)) == 0){
     if(timerPD2 == 0) {
       timerPD2 = millis();
     }
   } else {
     if(millis() - timerPD2 >= 2500 && timerPD2 != 0) {
        clearLeds(ledsP1Boats);
        clearLeds(ledsP2Boats);
        clearLeds(ledsP1Hits);
        clearLeds(ledsP2Hits);
     }
     timerPD2 = 0;
   }
}

long timerPD4 = 0;
ISR(PCINT2_vect) {
   if ((PIND & (1 << PD4)) == 0){
     if(timerPD4 == 0) {
       timerPD4 = millis();
     }
   } else {
     if(millis() - timerPD4 >= 2500 && timerPD4 != 0) {
        clearLeds(ledsP1Boats);
        clearLeds(ledsP2Boats);
        clearLeds(ledsP1Hits);
        clearLeds(ledsP2Hits);
     }
     timerPD4 = 0;
   }
}

int play(CRGB color, int SW_pin_joystick0, int X_pin_joystick0, int Y_pin_joystick0, CRGB *ledsP0, CRGB *hitsP0, CRGB *boatsEnemy);
void win(int player);
void start();

void loop() {
  start(); 
}

void setDisplay(int player) {
  int i;
  if(player == 1) {
    for(i = 0; i < 64; i++) {
      ledsP1[i] = ledsP1Hits[i];
      ledsP2[i] = ledsP2Boats[i];
    }
  } else {
    for(i = 0; i < 64; i++) {
      ledsP2[i] = ledsP2Hits[i];
      ledsP1[i] = ledsP1Boats[i];
    }
  }
  FastLED.show();
}

void debugBoats() {
  int i, j;
  for(i = 0; i < 8; i++) {
    for(j = 0; j < 8; j++) {
      if(ledsP1Boats[i * 8 + j] == empty) {
        Serial.print("#");
      } else {
        Serial.print(0);
      }
      Serial.print(" ");
    }
    Serial.print("\n");
  }
  Serial.print("\n");
  for(i = 0; i < 8; i++) {
    for(j = 0; j < 8; j++) {
      if(ledsP2Boats[i * 8 + j] == empty) {
        Serial.print("#");
      } else {
        Serial.print(0);
      }
      Serial.print(" ");
    }
    Serial.print("\n");
  }
}

void checkEnded() {
  int i, j;
  bool boatsRemain = false;
  for(i = 0; i < 8; i++) {
    for(j = 0; j < 8; j++) {
      if(ledsP1Boats[i * 8 + j] != empty) {
        boatsRemain = true;
        break;
      } 
    }
  }
  if(!boatsRemain) {
    //win player 2
    Serial.print("2 WON\n");
    win(2);
    resetGame();
    return;
  } 
  boatsRemain = false;
  for(i = 0; i < 8; i++) {
    for(j = 0; j < 8; j++) {
      if(ledsP2Boats[i * 8 + j] != empty) {
        boatsRemain = true;
        break;
      } 
    }
  }
  if(!boatsRemain) {
    //win player 1
    Serial.print("1 WON\n");
    win(1);
    resetGame();
    return;
  } 
}

void start() {
  setDisplay(1);
  int last = play(cPlayer0, SW_pin_joystick0, X_pin_joystick0, Y_pin_joystick0, ledsP1, ledsP1Hits, ledsP2Boats);
  
  checkEnded();

  setDisplay(2);
  last = play(cPlayer1, SW_pin_joystick1, X_pin_joystick1, Y_pin_joystick1, ledsP2, ledsP2Hits, ledsP1Boats);
  
  checkEnded();
  debugBoats();
  Serial.print("\n");

}

void win(int player) {
  clearLeds(ledsP1);
  clearLeds(ledsP2);

  ledsP1[1 * 8 + 2] = player % 2 ? CRGB::Green : CRGB::Blue;
  ledsP1[2 * 8 + 2] = player % 2 ? CRGB::Green : CRGB::Blue;
  ledsP1[1 * 8 + 5] = player % 2 ? CRGB::Green : CRGB::Blue;
  ledsP1[2 * 8 + 5] = player % 2 ? CRGB::Green : CRGB::Blue;

  ledsP2[1 * 8 + 2] = player % 2 ? CRGB::Blue : CRGB::Green;
  ledsP2[2 * 8 + 2] = player % 2 ? CRGB::Blue : CRGB::Green;
  ledsP2[1 * 8 + 5] = player % 2 ? CRGB::Blue : CRGB::Green;
  ledsP2[2 * 8 + 5] = player % 2 ? CRGB::Blue : CRGB::Green;

  ledsP1[5 * 8 + 2] = player % 2 ? CRGB::Green : CRGB::Blue;
  ledsP1[5 * 8 + 5] = player % 2 ? CRGB::Green : CRGB::Blue;
  ledsP2[5 * 8 + 2] = player % 2 ? CRGB::Blue : CRGB::Green;
  ledsP2[5 * 8 + 5] = player % 2 ? CRGB::Blue : CRGB::Green;

  if(player % 2 == 1) {
    ledsP1[4 * 8 + 1] = player % 2 ? CRGB::Green : CRGB::Blue;
    ledsP1[4 * 8 + 6] = player % 2 ? CRGB::Green : CRGB::Blue;
    ledsP2[6 * 8 + 1] = player % 2 ? CRGB::Blue : CRGB::Green;
    ledsP2[6 * 8 + 6] = player % 2 ? CRGB::Blue : CRGB::Green;

    ledsP1[6 * 8 + 3] = player % 2 ? CRGB::Green : CRGB::Blue;
    ledsP1[6 * 8 + 4] = player % 2 ? CRGB::Green : CRGB::Blue;
    ledsP2[4 * 8 + 3] = player % 2 ? CRGB::Blue : CRGB::Green;
    ledsP2[4 * 8 + 4] = player % 2 ? CRGB::Blue : CRGB::Green;
  } else {
    ledsP2[4 * 8 + 1] = player % 2 ? CRGB::Blue : CRGB::Green;
    ledsP2[4 * 8 + 6] = player % 2 ? CRGB::Blue : CRGB::Green;
    ledsP1[6 * 8 + 1] = player % 2 ? CRGB::Green : CRGB::Blue;
    ledsP1[6 * 8 + 6] = player % 2 ? CRGB::Green : CRGB::Blue;

    ledsP2[6 * 8 + 3] = player % 2 ? CRGB::Green : CRGB::Green;
    ledsP2[6 * 8 + 4] = player % 2 ? CRGB::Green : CRGB::Green;
    ledsP1[4 * 8 + 3] = player % 2 ? CRGB::Blue : CRGB::Blue;
    ledsP1[4 * 8 + 4] = player % 2 ? CRGB::Blue : CRGB::Blue;
  }

  FastLED.show();
  tone(buzzerPin, 250);
  delay(250);
  tone(buzzerPin, 450);
  delay(250);
  tone(buzzerPin, 250);
  delay(250);
  noTone(buzzerPin);
  while(1) {
    if(digitalRead(SW_pin_joystick0) == 0 || digitalRead(SW_pin_joystick1) == 0) {
      break;
    }
  }
}

int play(CRGB color, int SW_pin_joystick0, int X_pin_joystick0, int Y_pin_joystick0, CRGB *ledsP0, CRGB *hitsP0, CRGB *boatsEnemy) {
    FastLED.show();
    CRGB lastColor = CRGB::Black;
    while(1) { 
    if (analogRead(X_pin_joystick0) < 100) {
      lastCursorX = cursorX;
      lastCursorY = cursorY;
      lastColor = hitsP0[8 * lastCursorX + lastCursorY];

      if (cursorX > LIMIT_INF) {
        cursorX-=1;
      }
    } else 
    if (analogRead(Y_pin_joystick0) < 100) {
      lastCursorX = cursorX;
      lastCursorY = cursorY;
      lastColor = hitsP0[8 * lastCursorX + lastCursorY];
      if (cursorY < LIMIT_SUP) {
        cursorY+=1;
      }
    } else 

    if (analogRead(X_pin_joystick0) > 900){
      lastCursorX = cursorX;
      lastCursorY = cursorY;
      lastColor = hitsP0[8 * lastCursorX + lastCursorY];
      if (cursorX < LIMIT_SUP) {
        cursorX +=1;
      }
    } else
    if (analogRead(Y_pin_joystick0) > 900){
      lastCursorX = cursorX;
      lastCursorY = cursorY;
      lastColor = hitsP0[8 * lastCursorX + lastCursorY];
      if (cursorY > LIMIT_INF) {
        cursorY -=1;
      }
    }
    if (lastCursorX != cursorX || lastCursorY != cursorY) {
      ledsP0[lastCursorX * 8 + lastCursorY] = lastColor;
      ledsP0[cursorX * 8 + cursorY] = color;
      FastLED.show();
      delay(100);
    }
       
    if(digitalRead(SW_pin_joystick0) == 0){
        if(boatsEnemy[cursorX * 8 + cursorY] == empty) {
          hitsP0[cursorX * 8 + cursorY] = miss;
          tone(buzzerPin, 450);
          delay(300);
          noTone(buzzerPin);
        } else {
          hitsP0[cursorX * 8 + cursorY] = hit;
          boatsEnemy[cursorX * 8 + cursorY] = empty;
          tone(buzzerPin, 650);
          delay(300);
          noTone(buzzerPin);
        }
        FastLED.show();
        break;
      }
    }
}
